/*
    EIBD client library
    Copyright (C) 2005-2007 Martin Koegler <mkoegler@auto.tuwien.ac.at>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    In addition to the permissions in the GNU General Public License, 
    you may link the compiled version of this file into combinations
    with other programs, and distribute those combinations without any 
    restriction coming from the use of this file. (The General Public 
    License restrictions do apply in other respects; for example, they 
    cover modification of the file, and distribution when not linked into 
    a combine executable.)

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/







package tuwien.auto.eibclient;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Vector;

public class EIBConnection
{
  private int readlen = 0;
  private byte[] head = null;
  private byte[] data = null;
  private Socket con;
  private InputStream is;
  private OutputStream os;
  public EIBConnection (String host) throws IOException
  {
    this (host, 6720);
  }
  public EIBConnection (String host, int port) throws IOException
  {
    con = new Socket (host, port);
    is = con.getInputStream ();
    os = con.getOutputStream ();
    con.setTcpNoDelay (true);
  }
  protected int _EIB_SendRequest (byte[]data) throws IOException
  {
    if (is == null)
      throw new IOException ("connection closed");
    byte len[] = new byte[2];
    if (data.length > 0xffff || data.length < 2)
      {
	errno = EINVAL;
	return -1;
      }
    len[0] = (byte) ((data.length >> 8) & 0xff);
      len[1] = (byte) ((data.length) & 0xff);
      os.write (len);
      os.write (data);
      return 0;
  }
  protected int _EIB_CheckRequest (boolean block) throws IOException
  {
    int res;
    if (os == null)
      throw new IOException ("connection closed");
    if (readlen == 0)
      head = new byte[2];
    if (!block && is.available () < 1)
        return 0;
    if (readlen < 2)
      {
	res = is.read (head, readlen, 2 - readlen);
	if (res == -1)
	  {
	    throw new IOException ("connection closed");
	  }
	readlen += res;
	if (readlen < 2)
	  return 0;
	res =
	  (((((int) (head[0])) & 0xff) << 8) | ((((int) (head[1])) & 0xff)));
	data = new byte[res];
      }
    if (!block && is.available () < 1)
      return 0;
    res = is.read (data, readlen - 2, data.length + 2 - readlen);
    if (res == -1)
      {
	throw new IOException ("connection closed");
      }
    readlen += res;
    return 0;
  }
  protected int _EIB_GetRequest () throws IOException
  {
    do
      {
	if (_EIB_CheckRequest (true) == -1)
	  return -1;
      }
    while (readlen < 2 || (readlen >= 2 && readlen < data.length + 2));
    readlen = 0;
    return 0;
  }
  public int EIB_Poll_Complete () throws IOException
  {
    if (_EIB_CheckRequest (false) == -1)
      return -1;
    if (readlen < 2 || (readlen >= 2 && readlen < data.length + 2))
      return 0;
    return 1;
  }
  public int EIBClose () throws IOException
  {
    if (con == null)
      throw new IOException ("connection closed");
      try
    {
      con.close ();
    }
    finally
    {
      is = null;
      os = null;
      con = null;
    }
    return 0;
  }
  public int EIBClose_sync () throws IOException
  {
    try
    {
      EIBReset ();
    } catch (Exception e)
    {
    }
    return EIBClose ();
  }
  private Buffer buf;
  private Int16 ptr1;
  private Int8 ptr2;
  private Int8 ptr3;
  private Int16 ptr4;
  private EIBAddr ptr5;
  private EIBAddr ptr6;
  private int sendlen;
  private interface _complete
  {
    int complete () throws IOException;
  }
  private _complete complete = null;
  public int EIBComplete () throws IOException
  {
    return complete.complete ();
  }
  public static final int EINVAL = 1;
  public static final int ECONNRESET = 2;
  public static final int EBUSY = 3;
  public static final int EADDRINUSE = 4;
  public static final int ETIMEDOUT = 5;
  public static final int EADDRNOTAVAIL = 6;
  public static final int EIO = 7;
  public static final int EPERM = 8;
  public static final int ENOENT = 9;
  public static final int ENODEV = 10;
  private int errno = 0;
  public int getLastError ()
  {
    return errno;
  }
  public static final int IMG_UNKNOWN_ERROR = 0;
  public static final int IMG_UNRECOG_FORMAT = 1;
  public static final int IMG_INVALID_FORMAT = 2;
  public static final int IMG_NO_BCUTYPE = 3;
  public static final int IMG_UNKNOWN_BCUTYPE = 4;
  public static final int IMG_NO_CODE = 5;
  public static final int IMG_NO_SIZE = 6;
  public static final int IMG_LODATA_OVERFLOW = 7;
  public static final int IMG_HIDATA_OVERFLOW = 8;
  public static final int IMG_TEXT_OVERFLOW = 9;
  public static final int IMG_NO_ADDRESS = 10;
  public static final int IMG_WRONG_SIZE = 11;
  public static final int IMG_IMAGE_LOADABLE = 12;
  public static final int IMG_NO_DEVICE_CONNECTION = 13;
  public static final int IMG_MASK_READ_FAILED = 14;
  public static final int IMG_WRONG_MASK_VERSION = 15;
  public static final int IMG_CLEAR_ERROR = 16;
  public static final int IMG_RESET_ADDR_TAB = 17;
  public static final int IMG_LOAD_HEADER = 18;
  public static final int IMG_LOAD_MAIN = 19;
  public static final int IMG_ZERO_RAM = 20;
  public static final int IMG_FINALIZE_ADDR_TAB = 21;
  public static final int IMG_PREPARE_RUN = 22;
  public static final int IMG_RESTART = 23;
  public static final int IMG_LOADED = 24;
  public static final int IMG_NO_START = 25;
  public static final int IMG_WRONG_ADDRTAB = 26;
  public static final int IMG_ADDRTAB_OVERFLOW = 27;
  public static final int IMG_OVERLAP_ASSOCTAB = 28;
  public static final int IMG_OVERLAP_TEXT = 29;
  public static final int IMG_NEGATIV_TEXT_SIZE = 30;
  public static final int IMG_OVERLAP_PARAM = 31;
  public static final int IMG_OVERLAP_EEPROM = 32;
  public static final int IMG_OBJTAB_OVERFLOW = 33;
  public static final int IMG_WRONG_LOADCTL = 34;
  public static final int IMG_UNLOAD_ADDR = 35;
  public static final int IMG_UNLOAD_ASSOC = 36;
  public static final int IMG_UNLOAD_PROG = 37;
  public static final int IMG_LOAD_ADDR = 38;
  public static final int IMG_WRITE_ADDR = 39;
  public static final int IMG_SET_ADDR = 40;
  public static final int IMG_FINISH_ADDR = 41;
  public static final int IMG_LOAD_ASSOC = 42;
  public static final int IMG_WRITE_ASSOC = 43;
  public static final int IMG_SET_ASSOC = 44;
  public static final int IMG_FINISH_ASSOC = 45;
  public static final int IMG_LOAD_PROG = 46;
  public static final int IMG_ALLOC_LORAM = 47;
  public static final int IMG_ALLOC_HIRAM = 48;
  public static final int IMG_ALLOC_INIT = 49;
  public static final int IMG_ALLOC_RO = 50;
  public static final int IMG_ALLOC_EEPROM = 51;
  public static final int IMG_ALLOC_PARAM = 52;
  public static final int IMG_SET_PROG = 53;
  public static final int IMG_SET_TASK_PTR = 54;
  public static final int IMG_SET_OBJ = 55;
  public static final int IMG_SET_TASK2 = 56;
  public static final int IMG_FINISH_PROC = 57;
  public static final int IMG_WRONG_CHECKLIM = 58;
  public static final int IMG_INVALID_KEY = 59;
  public static final int IMG_AUTHORIZATION_FAILED = 60;
  public static final int IMG_KEY_WRITE = 61;


  private class EIBGetAPDU_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0025 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      i = data.length - 2;
        buf.data = new byte[i];
        System.arraycopy (data, 2, buf.data, 0, i);
        return i;
    }
  }
  public int EIBGetAPDU_async (Buffer buf) throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      this.buf = buf;
      complete = new EIBGetAPDU_complete ();
      return 0;
  }
  public int EIBGetAPDU (Buffer buf) throws IOException
  {
    if (EIBGetAPDU_async (buf) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBGetAPDU_Src_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0025 || data.length < 4)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      if (ptr5 != null)
	  ptr5.addr =
	  (short) (((((int) (data[2])) & 0xff) << 8) |
		   ((((int) (data[2 + 1])) & 0xff)));
        i = data.length - 4;
        buf.data = new byte[i];
        System.arraycopy (data, 4, buf.data, 0, i);
        return i;
    }
  }
  public int EIBGetAPDU_Src_async (Buffer buf, EIBAddr src) throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      this.buf = buf;
      this.ptr5 = src;
      complete = new EIBGetAPDU_Src_complete ();
      return 0;
  }
  public int EIBGetAPDU_Src (Buffer buf, EIBAddr src) throws IOException
  {
    if (EIBGetAPDU_Src_async (buf, src) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBGetBusmonitorPacket_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0014 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      i = data.length - 2;
        buf.data = new byte[i];
        System.arraycopy (data, 2, buf.data, 0, i);
        return i;
    }
  }
  public int EIBGetBusmonitorPacket_async (Buffer buf) throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      this.buf = buf;
      complete = new EIBGetBusmonitorPacket_complete ();
      return 0;
  }
  public int EIBGetBusmonitorPacket (Buffer buf) throws IOException
  {
    if (EIBGetBusmonitorPacket_async (buf) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBGetGroup_Src_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0027 || data.length < 6)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      if (ptr5 != null)
	  ptr5.addr =
	  (short) (((((int) (data[2])) & 0xff) << 8) |
		   ((((int) (data[2 + 1])) & 0xff)));
      if (ptr6 != null)
	  ptr6.addr =
	  (short) (((((int) (data[4])) & 0xff) << 8) |
		   ((((int) (data[4 + 1])) & 0xff)));
        i = data.length - 6;
        buf.data = new byte[i];
        System.arraycopy (data, 6, buf.data, 0, i);
        return i;
    }
  }
  public int EIBGetGroup_Src_async (Buffer buf, EIBAddr src,
				    EIBAddr dest) throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      this.buf = buf;
      this.ptr5 = src;
      this.ptr6 = dest;
      complete = new EIBGetGroup_Src_complete ();
      return 0;
  }
  public int EIBGetGroup_Src (Buffer buf, EIBAddr src,
			      EIBAddr dest) throws IOException
  {
    if (EIBGetGroup_Src_async (buf, src, dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBGetTPDU_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0025 || data.length < 4)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      if (ptr5 != null)
	  ptr5.addr =
	  (short) (((((int) (data[2])) & 0xff) << 8) |
		   ((((int) (data[2 + 1])) & 0xff)));
        i = data.length - 4;
        buf.data = new byte[i];
        System.arraycopy (data, 4, buf.data, 0, i);
        return i;
    }
  }
  public int EIBGetTPDU_async (Buffer buf, EIBAddr src) throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      this.buf = buf;
      this.ptr5 = src;
      complete = new EIBGetTPDU_complete ();
      return 0;
  }
  public int EIBGetTPDU (Buffer buf, EIBAddr src) throws IOException
  {
    if (EIBGetTPDU_async (buf, src) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_Cache_Clear_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0072 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_Cache_Clear_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0072 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0072) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_Cache_Clear_complete ();
      return 0;
  }
  public int EIB_Cache_Clear () throws IOException
  {
    if (EIB_Cache_Clear_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_Cache_Disable_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0071 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_Cache_Disable_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0071 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0071) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_Cache_Disable_complete ();
      return 0;
  }
  public int EIB_Cache_Disable () throws IOException
  {
    if (EIB_Cache_Disable_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_Cache_Enable_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0001)
	{
	  errno = EBUSY;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0070 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_Cache_Enable_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0070 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0070) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_Cache_Enable_complete ();
      return 0;
  }
  public int EIB_Cache_Enable () throws IOException
  {
    if (EIB_Cache_Enable_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_Cache_Read_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0075 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      if ((((((int) (data[4])) & 0xff) << 8) |
	   ((((int) (data[4 + 1])) & 0xff))) == 0)
	{
	  errno = ENODEV;
	  return -1;
	}
      if (data.length <= 6)
	{
	  errno = ENOENT;
	  return -1;
	}
      if (ptr5 != null)
	ptr5.addr =
	  (short) (((((int) (data[2])) & 0xff) << 8) |
		   ((((int) (data[2 + 1])) & 0xff)));
      i = data.length - 6;
      buf.data = new byte[i];
      System.arraycopy (data, 6, buf.data, 0, i);
      return i;
    }
  }
  public int EIB_Cache_Read_async (short dst, EIBAddr src,
				   Buffer buf) throws IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      this.buf = buf;
      this.ptr5 = src;
      ibuf[2] = (byte) ((dst >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dst) & 0xff);
      ibuf[0] = (byte) ((0x0075 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0075) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_Cache_Read_complete ();
      return 0;
  }
  public int EIB_Cache_Read (short dst, EIBAddr src,
			     Buffer buf) throws IOException
  {
    if (EIB_Cache_Read_async (dst, src, buf) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_Cache_Read_Sync_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0074 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      if ((((((int) (data[4])) & 0xff) << 8) |
	   ((((int) (data[4 + 1])) & 0xff))) == 0)
	{
	  errno = ENODEV;
	  return -1;
	}
      if (data.length <= 6)
	{
	  errno = ENOENT;
	  return -1;
	}
      if (ptr5 != null)
	ptr5.addr =
	  (short) (((((int) (data[2])) & 0xff) << 8) |
		   ((((int) (data[2 + 1])) & 0xff)));
      i = data.length - 6;
      buf.data = new byte[i];
      System.arraycopy (data, 6, buf.data, 0, i);
      return i;
    }
  }
  public int EIB_Cache_Read_Sync_async (short dst, EIBAddr src, Buffer buf,
					short age) throws IOException
  {
    byte head[] = new byte[6];
    byte ibuf[] = head;
      this.buf = buf;
      this.ptr5 = src;
      ibuf[2] = (byte) ((dst >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dst) & 0xff);
      ibuf[4] = (byte) ((age >> 8) & 0xff);
      ibuf[4 + 1] = (byte) ((age) & 0xff);
      ibuf[0] = (byte) ((0x0074 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0074) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_Cache_Read_Sync_complete ();
      return 0;
  }
  public int EIB_Cache_Read_Sync (short dst, EIBAddr src, Buffer buf,
				  short age) throws IOException
  {
    if (EIB_Cache_Read_Sync_async (dst, src, buf, age) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_Cache_Remove_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0073 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_Cache_Remove_async (short dest) throws IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[0] = (byte) ((0x0073 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0073) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_Cache_Remove_complete ();
      return 0;
  }
  public int EIB_Cache_Remove (short dest) throws IOException
  {
    if (EIB_Cache_Remove_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_LoadImage_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0063 || data.length < 4)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return (((((int) (data[2])) & 0xff) << 8) |
	      ((((int) (data[2 + 1])) & 0xff)));
  }}
  public int EIB_LoadImage_async (byte[]image) throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
    if (image.length < 0)
      throw new ArrayIndexOutOfBoundsException ("data to short");
      ibuf = new byte[head.length + image.length];
      sendlen = image.length;
      System.arraycopy (head, 0, ibuf, 0, head.length);
      System.arraycopy (image, 0, ibuf, head.length, image.length);
      ibuf[0] = (byte) ((0x0063 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0063) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_LoadImage_complete ();
      return 0;
  }
  public int EIB_LoadImage (byte[]image) throws IOException
  {
    if (EIB_LoadImage_async (image) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Authorize_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0057 || data.length < 3)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return (((int) (data[2])) & 0xff);
  }}
  public int EIB_MC_Authorize_async (byte[]key) throws IOException
  {
    byte head[] = new byte[6];
    byte ibuf[] = head;
    if (key.length != 4)
      throw new ArrayIndexOutOfBoundsException ("key is not 4 bytes long");
      System.arraycopy (key, 0, ibuf, 2, 4);
      ibuf[0] = (byte) ((0x0057 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0057) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Authorize_complete ();
      return 0;
  }
  public int EIB_MC_Authorize (byte[]key) throws IOException
  {
    if (EIB_MC_Authorize_async (key) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Connect_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0050 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_MC_Connect_async (short dest) throws IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[0] = (byte) ((0x0050 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0050) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Connect_complete ();
      return 0;
  }
  public int EIB_MC_Connect (short dest) throws IOException
  {
    if (EIB_MC_Connect_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_GetMaskVersion_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0059 || data.length < 4)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return (((((int) (data[2])) & 0xff) << 8) |
	      ((((int) (data[2 + 1])) & 0xff)));
  }}
  public int EIB_MC_GetMaskVersion_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0059 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0059) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_GetMaskVersion_complete ();
      return 0;
  }
  public int EIB_MC_GetMaskVersion () throws IOException
  {
    if (EIB_MC_GetMaskVersion_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_GetPEIType_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0055 || data.length < 4)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return (((((int) (data[2])) & 0xff) << 8) |
	      ((((int) (data[2 + 1])) & 0xff)));
  }}
  public int EIB_MC_GetPEIType_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0055 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0055) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_GetPEIType_complete ();
      return 0;
  }
  public int EIB_MC_GetPEIType () throws IOException
  {
    if (EIB_MC_GetPEIType_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Progmode_Off_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0060 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_MC_Progmode_Off_async () throws IOException
  {
    byte head[] = new byte[3];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((0) & 0xff);
      ibuf[0] = (byte) ((0x0060 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0060) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Progmode_Off_complete ();
      return 0;
  }
  public int EIB_MC_Progmode_Off () throws IOException
  {
    if (EIB_MC_Progmode_Off_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Progmode_On_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0060 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_MC_Progmode_On_async () throws IOException
  {
    byte head[] = new byte[3];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((1) & 0xff);
      ibuf[0] = (byte) ((0x0060 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0060) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Progmode_On_complete ();
      return 0;
  }
  public int EIB_MC_Progmode_On () throws IOException
  {
    if (EIB_MC_Progmode_On_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Progmode_Status_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0060 || data.length < 3)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return (((int) (data[2])) & 0xff);
  }}
  public int EIB_MC_Progmode_Status_async () throws IOException
  {
    byte head[] = new byte[3];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((3) & 0xff);
      ibuf[0] = (byte) ((0x0060 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0060) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Progmode_Status_complete ();
      return 0;
  }
  public int EIB_MC_Progmode_Status () throws IOException
  {
    if (EIB_MC_Progmode_Status_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Progmode_Toggle_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0060 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_MC_Progmode_Toggle_async () throws IOException
  {
    byte head[] = new byte[3];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((2) & 0xff);
      ibuf[0] = (byte) ((0x0060 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0060) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Progmode_Toggle_complete ();
      return 0;
  }
  public int EIB_MC_Progmode_Toggle () throws IOException
  {
    if (EIB_MC_Progmode_Toggle_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_PropertyDesc_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0061 || data.length < 6)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      if (ptr2 != null)
	  ptr2.data = data[2];
      if (ptr4 != null)
	  ptr4.data =
	  (short) (((((int) (data[3])) & 0xff) << 8) |
		   ((((int) (data[3 + 1])) & 0xff)));
      if (ptr3 != null)
	  ptr3.data = data[5];
        return 0;
    }
  }
  public int EIB_MC_PropertyDesc_async (byte obj, byte property, Int8 type,
					Int16 max_nr_of_elem,
					Int8 access) throws IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      this.ptr2 = type;
      this.ptr4 = max_nr_of_elem;
      this.ptr3 = access;
      ibuf[2] = (byte) ((obj) & 0xff);
      ibuf[3] = (byte) ((property) & 0xff);
      ibuf[0] = (byte) ((0x0061 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0061) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_PropertyDesc_complete ();
      return 0;
  }
  public int EIB_MC_PropertyDesc (byte obj, byte property, Int8 type,
				  Int16 max_nr_of_elem,
				  Int8 access) throws IOException
  {
    if (EIB_MC_PropertyDesc_async
	(obj, property, type, max_nr_of_elem, access) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_PropertyRead_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0053 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      i = data.length - 2;
        buf.data = new byte[i];
        System.arraycopy (data, 2, buf.data, 0, i);
        return i;
    }
  }
  public int EIB_MC_PropertyRead_async (byte obj, byte property, short start,
					byte nr_of_elem,
					Buffer buf) throws IOException
  {
    byte head[] = new byte[7];
    byte ibuf[] = head;
      this.buf = buf;
      ibuf[2] = (byte) ((obj) & 0xff);
      ibuf[3] = (byte) ((property) & 0xff);
      ibuf[4] = (byte) ((start >> 8) & 0xff);
      ibuf[4 + 1] = (byte) ((start) & 0xff);
      ibuf[6] = (byte) ((nr_of_elem) & 0xff);
      ibuf[0] = (byte) ((0x0053 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0053) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_PropertyRead_complete ();
      return 0;
  }
  public int EIB_MC_PropertyRead (byte obj, byte property, short start,
				  byte nr_of_elem,
				  Buffer buf) throws IOException
  {
    if (EIB_MC_PropertyRead_async (obj, property, start, nr_of_elem, buf) ==
	-1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_PropertyScan_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0062 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      i = data.length - 2;
        buf.data = new byte[i];
        System.arraycopy (data, 2, buf.data, 0, i);
        return i;
    }
  }
  public int EIB_MC_PropertyScan_async (Buffer buf) throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      this.buf = buf;
      ibuf[0] = (byte) ((0x0062 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0062) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_PropertyScan_complete ();
      return 0;
  }
  public int EIB_MC_PropertyScan (Buffer buf) throws IOException
  {
    if (EIB_MC_PropertyScan_async (buf) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_PropertyWrite_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0054 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      i = data.length - 2;
        buf.data = new byte[i];
        System.arraycopy (data, 2, buf.data, 0, i);
        return i;
    }
  }
  public int EIB_MC_PropertyWrite_async (byte obj, byte property, short start,
					 byte nr_of_elem, byte[]buf,
					 Buffer res) throws IOException
  {
    byte head[] = new byte[7];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((obj) & 0xff);
      ibuf[3] = (byte) ((property) & 0xff);
      ibuf[4] = (byte) ((start >> 8) & 0xff);
      ibuf[4 + 1] = (byte) ((start) & 0xff);
      ibuf[6] = (byte) ((nr_of_elem) & 0xff);
    if (buf.length < 0)
      throw new ArrayIndexOutOfBoundsException ("data to short");
      ibuf = new byte[head.length + buf.length];
      sendlen = buf.length;
      System.arraycopy (head, 0, ibuf, 0, head.length);
      System.arraycopy (buf, 0, ibuf, head.length, buf.length);
      this.buf = res;
      ibuf[0] = (byte) ((0x0054 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0054) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_PropertyWrite_complete ();
      return 0;
  }
  public int EIB_MC_PropertyWrite (byte obj, byte property, short start,
				   byte nr_of_elem, byte[]buf,
				   Buffer res) throws IOException
  {
    if (EIB_MC_PropertyWrite_async
	(obj, property, start, nr_of_elem, buf, res) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_ReadADC_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0056 || data.length < 4)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      if (ptr1 != null)
	  ptr1.data =
	  (short) (((((int) (data[2])) & 0xff) << 8) |
		   ((((int) (data[2 + 1])) & 0xff)));
        return 0;
    }
  }
  public int EIB_MC_ReadADC_async (byte channel, byte count,
				   Int16 val) throws IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      this.ptr1 = val;
      ibuf[2] = (byte) ((channel) & 0xff);
      ibuf[3] = (byte) ((count) & 0xff);
      ibuf[0] = (byte) ((0x0056 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0056) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_ReadADC_complete ();
      return 0;
  }
  public int EIB_MC_ReadADC (byte channel, byte count,
			     Int16 val) throws IOException
  {
    if (EIB_MC_ReadADC_async (channel, count, val) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Read_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0051 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      i = data.length - 2;
        buf.data = new byte[i];
        System.arraycopy (data, 2, buf.data, 0, i);
        return i;
    }
  }
  public int EIB_MC_Read_async (short addr, int buf_len,
				Buffer buf) throws IOException
  {
    byte head[] = new byte[6];
    byte ibuf[] = head;
      this.buf = buf;
      ibuf[2] = (byte) ((addr >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((addr) & 0xff);
      ibuf[4] = (byte) (((buf_len) >> 8) & 0xff);
      ibuf[4 + 1] = (byte) (((buf_len)) & 0xff);
      ibuf[0] = (byte) ((0x0051 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0051) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Read_complete ();
      return 0;
  }
  public int EIB_MC_Read (short addr, int buf_len,
			  Buffer buf) throws IOException
  {
    if (EIB_MC_Read_async (addr, buf_len, buf) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Restart_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x005a || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_MC_Restart_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x005a >> 8) & 0xff);
      ibuf[1] = (byte) ((0x005a) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Restart_complete ();
      return 0;
  }
  public int EIB_MC_Restart () throws IOException
  {
    if (EIB_MC_Restart_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_SetKey_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0002)
	{
	  errno = EPERM;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0058 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_MC_SetKey_async (byte[]key, byte level) throws IOException
  {
    byte head[] = new byte[7];
    byte ibuf[] = head;
    if (key.length != 4)
      throw new ArrayIndexOutOfBoundsException ("key is not 4 bytes long");
      System.arraycopy (key, 0, ibuf, 2, 4);
      ibuf[6] = (byte) ((level) & 0xff);
      ibuf[0] = (byte) ((0x0058 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0058) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_SetKey_complete ();
      return 0;
  }
  public int EIB_MC_SetKey (byte[]key, byte level) throws IOException
  {
    if (EIB_MC_SetKey_async (key, level) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Write_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0044)
	{
	  errno = EIO;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0052 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return sendlen;
    }
  }
  public int EIB_MC_Write_async (short addr, byte[]buf) throws IOException
  {
    byte head[] = new byte[6];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((addr >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((addr) & 0xff);
      ibuf[4] = (byte) (((buf.length) >> 8) & 0xff);
      ibuf[4 + 1] = (byte) (((buf.length)) & 0xff);
    if (buf.length < 0)
      throw new ArrayIndexOutOfBoundsException ("data to short");
      ibuf = new byte[head.length + buf.length];
      sendlen = buf.length;
      System.arraycopy (head, 0, ibuf, 0, head.length);
      System.arraycopy (buf, 0, ibuf, head.length, buf.length);
      ibuf[0] = (byte) ((0x0052 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0052) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Write_complete ();
      return 0;
  }
  public int EIB_MC_Write (short addr, byte[]buf) throws IOException
  {
    if (EIB_MC_Write_async (addr, buf) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_MC_Write_Plain_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x005b || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return sendlen;
    }
  }
  public int EIB_MC_Write_Plain_async (short addr,
				       byte[]buf) throws IOException
  {
    byte head[] = new byte[6];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((addr >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((addr) & 0xff);
      ibuf[4] = (byte) (((buf.length) >> 8) & 0xff);
      ibuf[4 + 1] = (byte) (((buf.length)) & 0xff);
    if (buf.length < 0)
      throw new ArrayIndexOutOfBoundsException ("data to short");
      ibuf = new byte[head.length + buf.length];
      sendlen = buf.length;
      System.arraycopy (head, 0, ibuf, 0, head.length);
      System.arraycopy (buf, 0, ibuf, head.length, buf.length);
      ibuf[0] = (byte) ((0x005b >> 8) & 0xff);
      ibuf[1] = (byte) ((0x005b) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_MC_Write_Plain_complete ();
      return 0;
  }
  public int EIB_MC_Write_Plain (short addr, byte[]buf) throws IOException
  {
    if (EIB_MC_Write_Plain_async (addr, buf) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_M_GetMaskVersion_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0031 || data.length < 4)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return (((((int) (data[2])) & 0xff) << 8) |
	      ((((int) (data[2 + 1])) & 0xff)));
  }}
  public int EIB_M_GetMaskVersion_async (short dest) throws IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[0] = (byte) ((0x0031 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0031) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_M_GetMaskVersion_complete ();
      return 0;
  }
  public int EIB_M_GetMaskVersion (short dest) throws IOException
  {
    if (EIB_M_GetMaskVersion_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_M_Progmode_Off_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0030 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_M_Progmode_Off_async (short dest) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[4] = (byte) ((0) & 0xff);
      ibuf[0] = (byte) ((0x0030 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0030) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_M_Progmode_Off_complete ();
      return 0;
  }
  public int EIB_M_Progmode_Off (short dest) throws IOException
  {
    if (EIB_M_Progmode_Off_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_M_Progmode_On_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0030 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_M_Progmode_On_async (short dest) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[4] = (byte) ((1) & 0xff);
      ibuf[0] = (byte) ((0x0030 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0030) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_M_Progmode_On_complete ();
      return 0;
  }
  public int EIB_M_Progmode_On (short dest) throws IOException
  {
    if (EIB_M_Progmode_On_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_M_Progmode_Status_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0030 || data.length < 3)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return (((int) (data[2])) & 0xff);
  }}
  public int EIB_M_Progmode_Status_async (short dest) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[4] = (byte) ((3) & 0xff);
      ibuf[0] = (byte) ((0x0030 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0030) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_M_Progmode_Status_complete ();
      return 0;
  }
  public int EIB_M_Progmode_Status (short dest) throws IOException
  {
    if (EIB_M_Progmode_Status_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_M_Progmode_Toggle_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0030 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_M_Progmode_Toggle_async (short dest) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[4] = (byte) ((2) & 0xff);
      ibuf[0] = (byte) ((0x0030 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0030) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_M_Progmode_Toggle_complete ();
      return 0;
  }
  public int EIB_M_Progmode_Toggle (short dest) throws IOException
  {
    if (EIB_M_Progmode_Toggle_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_M_ReadIndividualAddresses_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0032 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      i = data.length - 2;
        buf.data = new byte[i];
        System.arraycopy (data, 2, buf.data, 0, i);
        return i;
    }
  }
  public int EIB_M_ReadIndividualAddresses_async (Buffer buf) throws
    IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      this.buf = buf;
      ibuf[0] = (byte) ((0x0032 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0032) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_M_ReadIndividualAddresses_complete ();
      return 0;
  }
  public int EIB_M_ReadIndividualAddresses (Buffer buf) throws IOException
  {
    if (EIB_M_ReadIndividualAddresses_async (buf) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIB_M_WriteIndividualAddress_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0041)
	{
	  errno = EADDRINUSE;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0043)
	{
	  errno = ETIMEDOUT;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0042)
	{
	  errno = EADDRNOTAVAIL;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0040 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIB_M_WriteIndividualAddress_async (short dest) throws
    IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[0] = (byte) ((0x0040 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0040) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIB_M_WriteIndividualAddress_complete ();
      return 0;
  }
  public int EIB_M_WriteIndividualAddress (short dest) throws IOException
  {
    if (EIB_M_WriteIndividualAddress_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenBusmonitor_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0001)
	{
	  errno = EBUSY;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0010 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenBusmonitor_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0010 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0010) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenBusmonitor_complete ();
      return 0;
  }
  public int EIBOpenBusmonitor () throws IOException
  {
    if (EIBOpenBusmonitor_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenBusmonitorText_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0001)
	{
	  errno = EBUSY;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0011 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenBusmonitorText_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0011 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0011) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenBusmonitorText_complete ();
      return 0;
  }
  public int EIBOpenBusmonitorText () throws IOException
  {
    if (EIBOpenBusmonitorText_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpen_GroupSocket_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0026 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpen_GroupSocket_async (boolean write_only) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[4] = (byte) ((write_only) ? 0xff : 0);
      ibuf[0] = (byte) ((0x0026 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0026) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpen_GroupSocket_complete ();
      return 0;
  }
  public int EIBOpen_GroupSocket (boolean write_only) throws IOException
  {
    if (EIBOpen_GroupSocket_async (write_only) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenT_Broadcast_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0023 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenT_Broadcast_async (boolean write_only) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[4] = (byte) ((write_only) ? 0xff : 0);
      ibuf[0] = (byte) ((0x0023 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0023) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenT_Broadcast_complete ();
      return 0;
  }
  public int EIBOpenT_Broadcast (boolean write_only) throws IOException
  {
    if (EIBOpenT_Broadcast_async (write_only) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenT_Connection_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0020 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenT_Connection_async (short dest) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[0] = (byte) ((0x0020 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0020) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenT_Connection_complete ();
      return 0;
  }
  public int EIBOpenT_Connection (short dest) throws IOException
  {
    if (EIBOpenT_Connection_async (dest) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenT_Group_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0022 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenT_Group_async (short dest,
				   boolean write_only) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[4] = (byte) ((write_only) ? 0xff : 0);
      ibuf[0] = (byte) ((0x0022 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0022) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenT_Group_complete ();
      return 0;
  }
  public int EIBOpenT_Group (short dest,
			     boolean write_only) throws IOException
  {
    if (EIBOpenT_Group_async (dest, write_only) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenT_Individual_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0021 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenT_Individual_async (short dest,
					boolean write_only) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
      ibuf[4] = (byte) ((write_only) ? 0xff : 0);
      ibuf[0] = (byte) ((0x0021 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0021) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenT_Individual_complete ();
      return 0;
  }
  public int EIBOpenT_Individual (short dest,
				  boolean write_only) throws IOException
  {
    if (EIBOpenT_Individual_async (dest, write_only) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenT_TPDU_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0024 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenT_TPDU_async (short src) throws IOException
  {
    byte head[] = new byte[5];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((src >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((src) & 0xff);
      ibuf[0] = (byte) ((0x0024 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0024) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenT_TPDU_complete ();
      return 0;
  }
  public int EIBOpenT_TPDU (short src) throws IOException
  {
    if (EIBOpenT_TPDU_async (src) == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenVBusmonitor_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0001)
	{
	  errno = EBUSY;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0012 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenVBusmonitor_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0012 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0012) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenVBusmonitor_complete ();
      return 0;
  }
  public int EIBOpenVBusmonitor () throws IOException
  {
    if (EIBOpenVBusmonitor_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBOpenVBusmonitorText_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  == 0x0001)
	{
	  errno = EBUSY;
	  return -1;
	}
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0013 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBOpenVBusmonitorText_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0013 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0013) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBOpenVBusmonitorText_complete ();
      return 0;
  }
  public int EIBOpenVBusmonitorText () throws IOException
  {
    if (EIBOpenVBusmonitorText_async () == -1)
      return -1;
    return EIBComplete ();
  }

  private class EIBReset_complete implements _complete
  {
    public int complete () throws IOException
    {
      int i;
      if (_EIB_GetRequest () == -1)
	return -1;
      if (((((((int) (data[0])) & 0xff) << 8) | ((((int) (data[1])) & 0xff))))
	  != 0x0004 || data.length < 2)
	{
	  errno = ECONNRESET;
	  return -1;
	}
      return 0;
    }
  }
  public int EIBReset_async () throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
      ibuf[0] = (byte) ((0x0004 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0004) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      complete = new EIBReset_complete ();
      return 0;
  }
  public int EIBReset () throws IOException
  {
    if (EIBReset_async () == -1)
      return -1;
    return EIBComplete ();
  }

  public int EIBSendAPDU (byte[]data) throws IOException
  {
    byte head[] = new byte[2];
    byte ibuf[] = head;
    if (data.length < 2)
      throw new ArrayIndexOutOfBoundsException ("data to short");
      ibuf = new byte[head.length + data.length];
      sendlen = data.length;
      System.arraycopy (head, 0, ibuf, 0, head.length);
      System.arraycopy (data, 0, ibuf, head.length, data.length);
      ibuf[0] = (byte) ((0x0025 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0025) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      return 0;
  }

  public int EIBSendGroup (short dest, byte[]data) throws IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
    if (data.length < 2)
      throw new ArrayIndexOutOfBoundsException ("data to short");
      ibuf = new byte[head.length + data.length];
      sendlen = data.length;
      System.arraycopy (head, 0, ibuf, 0, head.length);
      System.arraycopy (data, 0, ibuf, head.length, data.length);
      ibuf[0] = (byte) ((0x0027 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0027) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      return 0;
  }

  public int EIBSendTPDU (short dest, byte[]data) throws IOException
  {
    byte head[] = new byte[4];
    byte ibuf[] = head;
      ibuf[2] = (byte) ((dest >> 8) & 0xff);
      ibuf[2 + 1] = (byte) ((dest) & 0xff);
    if (data.length < 2)
      throw new ArrayIndexOutOfBoundsException ("data to short");
      ibuf = new byte[head.length + data.length];
      sendlen = data.length;
      System.arraycopy (head, 0, ibuf, 0, head.length);
      System.arraycopy (data, 0, ibuf, head.length, data.length);
      ibuf[0] = (byte) ((0x0025 >> 8) & 0xff);
      ibuf[1] = (byte) ((0x0025) & 0xff);
    if (_EIB_SendRequest (ibuf) == -1)
      return -1;
      return 0;
  }
}
