/*
 * Copyright 2007 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 * Copyright 2005 Bernhard Erb
 * Copyright 2006 Jochen Burkhardt 
 * Copyright 2005-2006 Contributors

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

package tuwien.auto.eibdcl;

import tuwien.auto.eibclient.*;
import tuwien.auto.eicl.struct.cemi.EIB_Address;
import tuwien.auto.eibdcl.struct.GroupPacket;
import java.util.Vector;
import tuwien.auto.eibdcl.event.DisconnectEvent;
import tuwien.auto.eibdcl.event.EICLEventListener;
import tuwien.auto.eibdcl.event.FrameEvent;
import tuwien.auto.eicl.util.*;

public class GroupConnection implements Runnable {

    private Thread receiveThread;
    private Vector events = new Vector();

    private EIBConnection con = null;
    private boolean stop = false;

    public GroupConnection (String host, int port) throws EICLException
    {
	try {
	    EIBConnection c = new EIBConnection (host, port);
	    c.EIBOpen_GroupSocket(false);
	    con = c;
	    receiveThread = new Thread(this);
	    receiveThread.start();
	}
	catch(Exception e)
	    {
		throw new EICLException(e.getMessage());
	    }
    }

    public void send (EIB_Address addr, byte[] APDU) throws EICLException
    {
	try {
	int res = con.EIBSendGroup ((short)addr.getRawAddress(), APDU);
	if (res == -1)
	    throw new EICLException ("Send failed");
	} catch (java.io.IOException e) {
	    throw new EICLException(e.getMessage());
	}
    }


    private void cleanUp(String _Message) {
        //Object[] list = events.getListenerList();
        Object[] list = events.toArray();
        for (int i = 0; i < list.length; i++) {
            if (list[i] instanceof EICLEventListener)
                ((EICLEventListener) list[i]).serverDisconnected(new DisconnectEvent(this, _Message));
        }
    }


    public void disconnect(String _Message) throws EICLException {
	stop = true;
	receiveThread.stop();
	cleanUp(_Message);
	try {
	con.EIBClose();
	} catch (java.io.IOException e) {
	    throw new EICLException(e.getMessage());
	}
    }

    public void run() {
	while (!stop) {
	    try {
	    EIBAddr src = new EIBAddr();
	    EIBAddr dest = new EIBAddr();
	    Buffer buf = new Buffer();
	    int res = con.EIBGetGroup_Src(buf, src, dest);

	    if (res == -1) {
		    disconnect ("receive failed");
	    }
	    GroupPacket p = new GroupPacket (new EIB_Address(src.addr, false),new EIB_Address(dest.addr, true), buf.data);

	    FrameEvent event = new FrameEvent(this, p);
                                    
	    Object[] list = events.toArray();
	    for (int i = 0; i < list.length; i++) {
		if (list[i] instanceof EICLEventListener)
		    ((EICLEventListener) list[i]).newFrameReceived(event);
	    }

	    } catch (Exception e) {
		try {
		    disconnect (e.getMessage());
		} catch (Exception e1) {}
	    }
	}
    }

    /**
     * Adds a frame listener that is called on each incoming cEMI packets
     * 
     * @param eventListener
     */
    public void addFrameListener(EICLEventListener eventListener) {
        //events.add(eventListener.getClass(), eventListener);
        events.add(eventListener);
    }

    /**
     * Removes the frame listener
     * 
     * @param eventListener
     */
    public void removeFrameListener(EICLEventListener eventListener) {
        //events.remove(eventListener.getClass(), eventListener);
        events.add(eventListener);
    }



}
