/*
 * Copyright 2005 Bernhard Erb
 * Copyright 2005-2006 Contributors

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*
 $Log: PDUXlatorList.java,v $
 Revision 1.11  2006/08/14 15:08:23  gneug
 Typos and copy-paste errors in list of major DPTs corrected.
 Comments on previous revision: aggregate EIS types removed, IDs aligned with major DPT ordering in 3/7/2.

 Revision 1.10  2006/08/11 11:58:37  woif
 Datapointtype
 Some features added in EICLdemo

 Revision 1.9  2006/08/10 14:11:55  woif
 Bugfix: "Type not found" error corrected when writing 3BitControlled
 Corrected some implemenation flaws in 3BitControlled class

 Revision 1.8  2006/01/11 19:05:10  gneug
 PointPDUXlator_String added

 */

package tuwien.auto.eibxlator;

import tuwien.auto.eicl.util.*;

/**
 * <p>
 * The PDUXlatorList is a utility class of the eibxlator package. It is useful
 * to lookup for DPT major types, and accessing their corresponding minor types.
 * Using the getPointPDUXlator() method an instance of the respective eibxlator
 * is returned. Note that only the DPT types implemented in this version are
 * contained and therefore accessible in this list.
 * 
 * @author bernhard erb
 * @see tuwien.auto.eibxlator.PDUXlatorList
 * @see tuwien.auto.eibxlator.PointPDUXlator_2ByteFloat
 * @see tuwien.auto.eibxlator.PointPDUXlator_3BitControlled
 * @see tuwien.auto.eibxlator.PointPDUXlator_8BitUnsigned
 * @see tuwien.auto.eibxlator.PointPDUXlator_Boolean
 * @see tuwien.auto.eibxlator.PointPDUXlator_String
 * @see tuwien.auto.eibxlator.PointPDUXlator
 */

public class PDUXlatorList {
    // Parameter 2 indicates the corresponding class implementation.
    /**
     * DPT type Boolean
     */
    public static final String[] TYPE_BOOLEAN = { "0x01", "Boolean (DPT 1)",
            "tuwien.auto.eibxlator.PointPDUXlator_Boolean" };
    
    /**
     * DPT type 1Bit Controlled
     */
    public static final String[] TYPE_1BIT_CONTROLLED = { "0x02", "1-Bit controlled (DPT 2)",
            "not implemented" };
    
    /**
     * DPT type 3Bit Controlled
     */
    public static final String[] TYPE_3BIT_CONTROLLED = { "0x03", "3-Bit controlled (DPT 3)",
            "tuwien.auto.eibxlator.PointPDUXlator_3BitControlled" };

    
    /**
     * DPT type Character Set
     */
    public static final String[] TYPE_CHARACTER_SET = { "0x04", "Character set (DPT 4)", 
            "not implemented" };
    
    /**
     * DPT type 8-Bit unsigned
     */
    public static final String[] TYPE_8BIT_UNSIGNED = { "0x05", "8-Bit unsigned (DPT 5)",
            "tuwien.auto.eibxlator.PointPDUXlator_8BitUnsigned" };
    
    /**
     * DPT type 8-Bit signed (shares a major DPT number with "Status with mode") 
     */
    public static final String[] TYPE_8BIT_SIGNED = { "0x06", "8-Bit signed (DPT 6)",
            "not implemented"};
    
    /**
     * DPT type Status with mode (shares a major DPT number with "8-Bit signed")
     */
    public static final String[] TYPE_STATUS_WITH_MODE = { "0x07", "Status with mode (DPT 6)",
            "not implemented"};
    
    /**
     * DPT type 2-octet unsigned
     */
    public static final String[] TYPE_2OCTET_UNSIGNED = { "0x08", "2-Octet unsigned (DPT 7)",
            "not implemented"};
    
    /**
     * DPT type 2-octet signed
     */
    public static final String[] TYPE_2OCTET_SIGNED = { "0x09", "2-Octet signed (DPT 8)",
            "not implemented"};
    
    /**
     * DPT type 2-octet Float
     */
    public static final String[] TYPE_2OCTET_FLOAT = { "0x10", "2-Octet Float Value (DPT 9)",
            "tuwien.auto.eibxlator.PointPDUXlator_2ByteFloat" };
    
    /**
     * DPT type Time
     */
    public static final String[] TYPE_TIME = { "0x11", "Time (DPT 10)",
            "not implemented"};
    
    
    /**
     * DPT type Date
     */
    public static final String[] TYPE_DATE = { "0x12", "Date (DPT 11)",
            "not implemented"};

    
    /**
     * DPT type 4 octet unsigned
     */
    public static final String[] TYPE_4OCTET_UNSIGNED = { "0x13", "4-Octet unsigned (DPT 12)",
            "not implemented"};
    
    /**
     * DPT type 4 octet signed
     */
    public static final String[] TYPE_4OCTET_SIGNED = { "0x14", "4-Octet signed (DPT 13)",
            "not implemented"};
    
    /**
     * DPT type 4 octet Float
     */
    public static final String[] TYPE_4OCTET_FLOAT = { "0x15", "4-Octet Float Value (DPT 14)",
            "not implemented" };
    
    /**
     * DPT type Access
     */
    public static final String[] TYPE_ACCESS = { "0x16", "Access (DPT 15)",
           "not implemented"};
    
    /**
     * DPT type string
     */
    public static final String[] TYPE_STRING = { "0x17", "String (DPT 16)",
            "tuwien.auto.eibxlator.PointPDUXlator_String" };


    private static EICLMap eiclMap = null;

    private static void initEICLMap() {
        eiclMap = new EICLMap(); // fillHashtable
       
        eiclMap.put(TYPE_BOOLEAN[0], TYPE_BOOLEAN);
        eiclMap.put(TYPE_3BIT_CONTROLLED[0], TYPE_3BIT_CONTROLLED);
        eiclMap.put(TYPE_8BIT_UNSIGNED[0], TYPE_8BIT_UNSIGNED);
        eiclMap.put(TYPE_2OCTET_FLOAT[0], TYPE_2OCTET_FLOAT);
        eiclMap.put(TYPE_STRING[0], TYPE_STRING);
        
    }

    /**
     * Returns the map of all implemented Major types
     * 
     * @return The major type map.
     */
    public static EICLMap getPointTypeMajor() {
        if (eiclMap == null)
            initEICLMap();
        return eiclMap;
    }

    /**
     * Returns an PDU translator instance for the requested major and minor type .
     * 
     * @param _MajorTypeKey
     *            The requested major type
     * @param _MinorTypeKey
     *            The requested minor type
     * @return An instance of the to the major and minor type corresponding
     *         translator
     * @throws EICLException
     *             If either the major or minor type can not be found.
     */
    public static PointPDUXlator getPointPDUXlator(String _MajorTypeKey, String _MinorTypeKey)
            throws EICLException {
        if (eiclMap == null)
            initEICLMap();
        String[] val = eiclMap.getTypeValues(_MajorTypeKey);
        PointPDUXlator xlator = null;
        try {
            Class[] c = new Class[] { String.class };
            xlator = (PointPDUXlator) Class.forName(val[0]).getConstructor(c).newInstance(
                    new Object[] { _MinorTypeKey });
        } catch (Exception ex) {
            throw new EICLException("Unimplemented DPT");
        }
        return xlator;
    }

    /**
     * Returns a list containing all known minor types of the requested major
     * type.
     * 
     * @param _MajorKey
     *            The major type
     * @return The map containing the corresponding minor types.
     * @throws EICLException
     *             If the major type can't be found.
     */
    public static EICLMap getPointPDUMinor(String _MajorKey) throws EICLException {
        if (eiclMap == null)
            initEICLMap();
        EICLMap ret = null;
        String[] val = eiclMap.getTypeValues(_MajorKey);
        try {
            PointPDUXlator xlator = (PointPDUXlator) Class.forName(val[0]).newInstance();
            ret = xlator.getMinorTypes();
        } catch (ClassNotFoundException ex) {
            throw new EICLException(ex.getMessage());
        } catch (ExceptionInInitializerError ex) {
            throw new EICLException(ex.getMessage());
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new EICLException(ex.getMessage());
        }
        return ret;
    }

}
